{% import "WebPortfolio/macros/base.html" as m with context %}
{% import "WebPortfolio/macros/forms.html" as f with context %}
{% extends "WebPortfolio/Module/Cms/Admin/_layout.html" %}

{% block admin_content %}

    <div class="container preview">

        <div class="row">
            <div class="col-md-12">
                    <a href="{{ url_for("CmsAdmin:index") }}" class="btn btn-default"><i class="fa fa-arrow-left"></i></a>
                    <a href="{{ url_for("CmsAdmin:edit", id=post.id) }}" class="btn btn-primary"><i class="fa fa-pencil"></i> Edit</a>
            </div>
        </div>

        <div class="row">
            <div class="col-md-12">
                <h1 class="title">{{ post.title }}</h1>
            </div>
        </div>

        <div class="row">
            <div class="col-md-12">
                <div class="content">{{ post.content | markdown | safe }}</div>
            </div>
        </div>
    </div>


{% endblock %}