{#
Navigation: Breadcrum
#}

{% macro menu(key) %}

    <ol class="breadcrumb">
        {% for nav in __g.NAV_MENU %}
            {% set nav_name = nav[1] %}
            {% set nav_items = nav[2] %}
            {% set nav_kwargs = nav[3] %}

            {% if nav_kwargs["key"] == key %}
                {% for item in nav_items %}
                    {% set item_name = item[1] %}
                    {% set item_endpoint = item[2] %}
                    {% set item_kwargs = item[3] %}

                    <li><a href="{{ url_for(item_endpoint) }}">{{ item_name }}</a></li>
                {% endfor %}
            {% endif %}
        {% endfor %}
    </ol>

{% endmacro %}
