
{% import "WebPortfolio/macros/base.html" as  wp with context %}
{% import "WebPortfolio/macros/navbar.html" as navbar with context %}
{% import "WebPortfolio/macros/bootstrap.html" as bs with context %}

<!DOCTYPE html>
<html>

<head>

    {{ wp.site_header("%s - Admin", add_social_graph=false, add_google_analytics=false) }}

    {{ wp.css_assets("admin-all-css") }}

    {{ bs.theme('flatly') }}

</head>
<body>

    {{ navbar.navbar(brand="My Site Admin", groups=["admin", "account"]) }}

    <div class="container-fluid">

        <div class="row">
            <div class="col-md-12">
                {{ wp.flashed_messages() }}
            </div>
        </div>

        {# The content of the site will be included here #}
        {% include __template__ %}

    </div>

    {{ wp.js_assets("admin-all-js") }}

</body>
</html>



