# About

name = "WebPortfolio"
title = "WebPortfolio"
version = "0.0.6"
author = "Mardix"
email = "mardix@pylot.io"
description = "A full stack Flask based framework that put structure in the " \
              "file system. Features: Caching, Mailing, Storage, CSRF, recaptcha " \
              "and more"
uri = "https://github.com/mardix/web-portfolio"
license = "MIT"
copyright = "(c) 2015 %s" % author