<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
    <meta http-equiv="Content-type" content="text/html;charset=UTF-8">
    <title>Site Maintenance</title>
    <link href='//fonts.googleapis.com/css?family=Raleway:400,300,600' rel='stylesheet' type='text/css'>
    <link rel="stylesheet" href="//maxcdn.bootstrapcdn.com/bootstrap/3.3.4/css/bootstrap.min.css">
    <style type="text/css">
      body { font: 20px "Raleway", Helvetica, sans-serif; color: #666; text-align: center; padding-top: 150px;}
        h1 { font-size: 50px; color: #333; }
    </style>
</head>
<body>
    <div class="container">
        <div class="row">
            <div class="col-md-12 maintenance">
                <h1>We'll be back soon!</h1>
                <div>
                   Sorry for the inconvenience but we're performing some maintenance at the moment. <br>
                    We'll be back online shortly. <br>
                    Thanks!
                </div>
            </div>
        </div>
    </div>
</body>
</html>